@@ -23,8 +23,25 @@ ver="`latest_automake`"
 PQXXVERSION=`./tools/extract_version`
 echo "libpqxx version $PQXXVERSION"
 
+PQXX_ABI=`./tools/extract_version --abi`
+echo "libpqxx ABI version $PQXX_ABI"
+
+PQXX_MAJOR="`echo "$PQXXVERSION" | sed -e 's/[[:space:]]*\([0-9]*\)\..*$/\1/'`"
+PQXX_MINOR="`echo "$PQXXVERSION" | sed -e 's/[^.]*\.\([0-9]*\).*$/\1/'`"
+
+substitute() {
+	sed -e "s/@PQXXVERSION@/$PQXXVERSION/g" \
+		-e "s/@PQXX_MAJOR@/$PQXX_MAJOR/g" \
+		-e "s/@PQXX_MINOR@/$PQXX_MINOR/g" \
+		-e "s/@PQXX_ABI@/$PQXX_ABI/g" \
+		"$1"
+}
+
 # Generate configure.ac based on current version numbers
-sed -e "s/@PQXXVERSION@/$PQXXVERSION/g" configure.ac.in >configure.ac
+substitute configure.ac.in >configure.ac
+
+# Generate version header.
+substitute include/pqxx/version.hxx.template >include/pqxx/version.hxx
 
 # Generate Windows makefiles (adding carriage returns to make it MS-DOS format)
 makewinmake() {
