@@ -7,7 +7,7 @@
  *      implementation of the pqxx::connection_base abstract base class.
  *   pqxx::connection_base encapsulates a frontend to backend connection
  *
- * Copyright (c) 2001-2007, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -21,6 +21,7 @@
 #include <cassert>
 #include <cerrno>
 #include <cstdio>
+#include <cstring>
 #include <ctime>
 #include <stdexcept>
 
@@ -49,7 +50,7 @@
 #include "pqxx/pipeline"
 #include "pqxx/result"
 #include "pqxx/transaction"
-#include "pqxx/trigger"
+#include "pqxx/notify-listen"
 
 using namespace PGSTD;
 using namespace pqxx;
@@ -81,8 +82,22 @@ static fd_set *const fdset_none = &emptyfd;
 // by using "C" linkage, perhaps silence them altogether.
 static void set_fdbit(int f, fd_set *s)
 {
+
+#ifdef _MSC_VER
+#pragma warning ( push )
+#pragma warning ( disable: 4389 ) // signed/unsigned mismatch
+#pragma warning ( disable: 4127 ) // conditional expression is constant
+#endif
+
   FD_SET(f, s);
+
+#ifdef _MSC_VER
+#pragma warning ( pop )
+#endif
+
 }
+
+
 static void clear_fdmask(fd_set *mask)
 {
   FD_ZERO(mask);
@@ -149,7 +164,7 @@ int pqxx::connection_base::sock() const throw ()
 
 void pqxx::connection_base::activate()
 {
-  if (!is_open()) try
+  if (!is_open())
   {
     if (m_inhibit_reactivation)
       throw broken_connection("Could not reactivate connection; "
@@ -159,25 +174,27 @@ void pqxx::connection_base::activate()
     // connection, don't try to reactivate
     if (m_reactivation_avoidance.get()) return;
 
-    m_Conn = m_policy.do_startconnect(m_Conn);
-    m_Conn = m_policy.do_completeconnect(m_Conn);
-    m_Completed = true;	// (But retracted if error is thrown below)
+    try
+    {
+      m_Conn = m_policy.do_startconnect(m_Conn);
+      m_Conn = m_policy.do_completeconnect(m_Conn);
+      m_Completed = true;	// (But retracted if error is thrown below)
 
-    if (!is_open()) throw broken_connection();
+      if (!is_open()) throw broken_connection();
 
-    SetupState();
-  }
-  catch (const broken_connection &e)
-  {
-    const string Msg( ErrMsg() );
-    disconnect();
-    m_Completed = false;
-    throw broken_connection(e.what());
-  }
-  catch (const exception &)
-  {
-    m_Completed = false;
-    throw;
+      SetupState();
+    }
+    catch (const broken_connection &e)
+    {
+      disconnect();
+      m_Completed = false;
+      throw broken_connection(e.what());
+    }
+    catch (const exception &)
+    {
+      m_Completed = false;
+      throw;
+    }
   }
 }
 
@@ -187,7 +204,7 @@ void pqxx::connection_base::deactivate()
   if (!m_Conn) return;
 
   if (m_Trans.get())
-    throw logic_error("Attempt to deactivate connection while " +
+    throw usage_error("Attempt to deactivate connection while " +
 	m_Trans.get()->description() + " still open");
 
   if (m_reactivation_avoidance.get())
@@ -281,12 +298,12 @@ void pqxx::connection_base::SetupState()
   {
     const string Msg( ErrMsg() );
     m_Conn = m_policy.do_disconnect(m_Conn);
-    throw runtime_error(Msg);
+    throw failure(Msg);
   }
 
   read_capabilities();
 
-  const PSMap::const_iterator prepared_end(m_prepared.end());
+  PSMap::iterator prepared_end(m_prepared.end());
   for (PSMap::iterator p = m_prepared.begin(); p != prepared_end; ++p)
     p->second.registered = false;
 
@@ -295,21 +312,21 @@ void pqxx::connection_base::SetupState()
 
   InternalSetTrace();
 
-  if (!m_Triggers.empty() || !m_Vars.empty())
+  if (!m_listeners.empty() || !m_Vars.empty())
   {
     stringstream restore_query;
 
-    // Pipeline all queries needed to restore triggers and variables, so we can
+    // Pipeline all queries needed to restore listeners and variables, so we can
     // send them over in one go.
 
-    // Reinstate all active triggers
-    if (!m_Triggers.empty())
+    // Reinstate all active listeners
+    if (!m_listeners.empty())
     {
-      const TriggerList::const_iterator End = m_Triggers.end();
+      const listenerlist::const_iterator End = m_listeners.end();
       string Last;
-      for (TriggerList::const_iterator i = m_Triggers.begin(); i != End; ++i)
+      for (listenerlist::const_iterator i = m_listeners.begin(); i != End; ++i)
       {
-        // m_Triggers can handle multiple Triggers waiting on the same event;
+        // m_listeners can handle multiple listeners waiting on the same event;
         // issue just one LISTEN for each event.
         if (i->first != Last)
         {
@@ -327,7 +344,10 @@ void pqxx::connection_base::SetupState()
     PQsendQuery(m_Conn, restore_query.str().c_str());
     result r;
     const int proto = protocol_version();
-    do r = result(PQgetResult(m_Conn), proto, "[RECONNECT]"); while (r);
+    const int encoding = encoding_code();
+    do
+      r = result(PQgetResult(m_Conn), proto, "[RECONNECT]", encoding);
+    while (r);
   }
 
   m_Completed = true;
@@ -340,7 +360,7 @@ void pqxx::connection_base::check_result(const result &R)
   if (!is_open()) throw broken_connection();
 
   // A shame we can't quite detect out-of-memory to turn this into a bad_alloc!
-  if (!R) throw runtime_error(ErrMsg());
+  if (!R) throw failure(ErrMsg());
 
   try
   {
@@ -551,62 +571,65 @@ void pqxx::connection_base::trace(FILE *Out) throw ()
 }
 
 
-void pqxx::connection_base::AddTrigger(pqxx::trigger *T)
+void pqxx::connection_base::add_listener(pqxx::notify_listener *T)
 {
-  if (!T) throw invalid_argument("Null trigger registered");
+  if (!T) throw argument_error("Null listener registered");
 
-  // Add to triggers list and attempt to start listening.
-  const TriggerList::iterator p = m_Triggers.find(T->name());
-  const TriggerList::value_type NewVal(T->name(), T);
+  // Add to listener list and attempt to start listening.
+  const listenerlist::iterator p = m_listeners.find(T->name());
+  const listenerlist::value_type NewVal(T->name(), T);
 
-  if (p == m_Triggers.end())
+  if (p == m_listeners.end())
   {
     // Not listening on this event yet, start doing so.
     const string LQ("LISTEN \"" + T->name() + "\"");
 
     if (is_open()) try
     {
-      result R( PQexec(m_Conn, LQ.c_str()), protocol_version(), LQ );
+      result R( PQexec(m_Conn,
+		LQ.c_str()),
+		protocol_version(),
+		LQ,
+		encoding_code() );
       check_result(R);
     }
     catch (const broken_connection &)
     {
     }
-    m_Triggers.insert(NewVal);
+    m_listeners.insert(NewVal);
   }
   else
   {
-    m_Triggers.insert(p, NewVal);
+    m_listeners.insert(p, NewVal);
   }
-
 }
 
 
-void pqxx::connection_base::RemoveTrigger(pqxx::trigger *T) throw ()
+void pqxx::connection_base::remove_listener(pqxx::notify_listener *T) throw ()
 {
   if (!T) return;
 
   try
   {
     // Keep Sun compiler happy...  Hope it doesn't annoy other compilers
-    pair<const string, trigger *> tmp_pair(T->name(), T);
-    TriggerList::value_type E = tmp_pair;
+    pair<const string, notify_listener *> tmp_pair(T->name(), T);
+    listenerlist::value_type E = tmp_pair;
 
-    typedef pair<TriggerList::iterator, TriggerList::iterator> Range;
-    Range R = m_Triggers.equal_range(E.first);
+    typedef pair<listenerlist::iterator, listenerlist::iterator> Range;
+    Range R = m_listeners.equal_range(E.first);
 
-    const TriggerList::iterator i = find(R.first, R.second, E);
+    const listenerlist::iterator i = find(R.first, R.second, E);
 
     if (i == R.second)
     {
-      process_notice("Attempt to remove unknown trigger '" + E.first + "'");
+      process_notice("Attempt to remove unknown listener '" + E.first + "'");
     }
     else
     {
-      // Erase first; otherwise a notification for the same trigger may yet come
-      // in and wreak havoc.  Thanks Dragan Milenkovic.
+      // Erase first; otherwise a notification for the same listener may yet
+      // come in and wreak havoc.  Thanks Dragan Milenkovic.
       const bool gone = (m_Conn && (R.second == ++R.first));
-      m_Triggers.erase(i);
+      m_listeners.erase(i);
       if (gone) Exec(("UNLISTEN \"" + T->name() + "\"").c_str(), 0);
     }
   }
@@ -629,12 +652,49 @@ bool pqxx::connection_base::is_busy() const throw ()
 }
 
 
+#ifdef PQXX_HAVE_PQCANCEL
+namespace
+{
+class cancel_wrapper
+{
+  PGcancel *m_cancel;
+  char m_errbuf[500];
+public:
+  cancel_wrapper(PGconn *conn) :
+    m_cancel(NULL),
+    m_errbuf()
+  {
+    m_cancel = PQgetCancel(conn);
+    if (!m_cancel) throw bad_alloc();
+  }
+  ~cancel_wrapper() { PQfreeCancel(m_cancel); }
+
+  void operator()()
+  {
+    if (!PQcancel(m_cancel, m_errbuf, int(sizeof(m_errbuf))))
+      throw sql_error(string(m_errbuf));
+  }
+};
+}
+#endif
+
+void pqxx::connection_base::cancel_query()
+{
+#ifdef PQXX_HAVE_PQCANCEL
+  cancel_wrapper cancel(m_Conn);
+  cancel();
+#else
+  if (!PQrequestCancel(m_Conn)) throw sql_error(ErrMsg());
+#endif
+}
+
+
 int pqxx::connection_base::get_notifs()
 {
   int notifs = 0;
   if (!is_open()) return notifs;
 
-  PQconsumeInput(m_Conn);
+  if (!consume_input()) throw broken_connection();
 
   // Even if somehow we receive notifications during our transaction, don't
   // deliver them.
@@ -642,11 +702,11 @@ int pqxx::connection_base::get_notifs()
 
   for (PQAlloc<PGnotify> N( PQnotifies(m_Conn) ); N; N = PQnotifies(m_Conn))
   {
-    typedef TriggerList::iterator TI;
+    typedef listenerlist::iterator TI;
 
     notifs++;
 
-    pair<TI, TI> Hit = m_Triggers.equal_range(string(N->relname));
+    pair<TI, TI> Hit = m_listeners.equal_range(string(N->relname));
     for (TI i = Hit.first; i != Hit.second; ++i) try
     {
       (*i->second)(N->be_pid);
@@ -655,7 +715,7 @@ int pqxx::connection_base::get_notifs()
     {
       try
       {
-        process_notice("Exception in trigger handler '" +
+        process_notice("Exception in notification listener '" +
 		       i->first +
 		       "': " +
 		       e.what() +
@@ -664,12 +724,12 @@ int pqxx::connection_base::get_notifs()
       catch (const bad_alloc &)
       {
         // Out of memory.  Try to get the message out in a more robust way.
-        process_notice("Exception in trigger handler, "
+        process_notice("Exception in notification listener, "
 	    "and also ran out of memory\n");
       }
       catch (const exception &)
       {
-        process_notice("Exception in trigger handler "
+        process_notice("Exception in notification listener "
 	    "(compounded by other error)\n");
       }
     }
@@ -718,13 +778,17 @@ pqxx::result pqxx::connection_base::Exec(const char Query[], int Retries)
 {
   activate();
 
-  result R(PQexec(m_Conn, Query), protocol_version(), Query);
+  result R(PQexec(m_Conn, Query), protocol_version(), Query, encoding_code());
 
   while ((Retries > 0) && !R && !is_open())
   {
     Retries--;
     Reset();
-    if (is_open()) R = result(PQexec(m_Conn, Query), protocol_version(), Query);
+    if (is_open())
+      R = result(PQexec(m_Conn, Query),
+	protocol_version(),
+	Query,
+	encoding_code());
   }
 
   check_result(R);
@@ -742,7 +806,7 @@ pqxx::prepare::declaration pqxx::connection_base::prepare(
   if (i != m_prepared.end())
   {
     if (definition != i->second.definition)
-      throw invalid_argument("Inconsistent redefinition "
+      throw argument_error("Inconsistent redefinition "
 	  "of prepared statement " + name);
 
     // Prepare for repeated repetition of parameters
@@ -773,7 +837,8 @@ void pqxx::connection_base::unprepare(const PGSTD::string &name)
 
 namespace
 {
-string escape_param(const char in[],
+string escape_param(connection_base &C,
+	const char in[],
 	int len,
 	prepare::param_treatment treatment)
 {
@@ -785,7 +850,7 @@ string escape_param(const char in[],
     return "'" + escape_binary(string(in,len)) + "'";
 
   case treat_string:
-    return "'" + pqxx::internal::escape_string(in, strlen(in)) + "'";
+    return C.quote(in);
 
   case treat_bool:
     switch (in[0])
@@ -798,7 +863,7 @@ string escape_param(const char in[],
     default:
       {
         // Looks like a numeric value may have been passed.  Try to convert it
-        // back to a number, then to bool to normalize its representation
+        // back to a number, then to bool to normalize its representation.
         bool b;
 	from_string(in,b);
         return to_string(b);
@@ -810,7 +875,7 @@ string escape_param(const char in[],
     break;
 
   default:
-    throw logic_error("Unknown treatment for prepared-statement parameter");
+    throw usage_error("Unknown treatment for prepared-statement parameter");
   }
 
   return in;
@@ -823,7 +888,7 @@ pqxx::connection_base::find_prepared(const PGSTD::string &statement)
 {
   PSMap::iterator s = m_prepared.find(statement);
   if (s == m_prepared.end())
-    throw invalid_argument("Unknown prepared statement '" + statement + "'");
+    throw argument_error("Unknown prepared statement '" + statement + "'");
   return s->second;
 }
 
@@ -834,7 +899,7 @@ void pqxx::connection_base::prepare_param_declare(
 {
   prepare::internal::prepared_def &s = find_prepared(statement);
   if (s.complete)
-    throw logic_error("Attempt to add parameter to prepared statement " +
+    throw usage_error("Attempt to add parameter to prepared statement " +
 	statement +
 	" after its definition was completed");
   s.addparam(sqltype,treatment);
@@ -858,7 +923,8 @@ pqxx::connection_base::register_prepared(const PGSTD::string &name)
     {
       result r(PQprepare(m_Conn, name.c_str(), s.definition.c_str(), 0, 0),
 	protocol_version(),
-	"[PREPARE " + name + "]");
+	"[PREPARE " + name + "]",
+	encoding_code());
       check_result(r);
       s.registered = true;
       return s;
@@ -899,7 +965,7 @@ pqxx::result pqxx::connection_base::prepared_exec(
   prepare::internal::prepared_def &s = register_prepared(statement);
 
   if (nparams != int(s.parameters.size()))
-    throw logic_error("Wrong number of parameters for prepared statement " +
+    throw usage_error("Wrong number of parameters for prepared statement " +
 	statement + ": "
 	"expected " + to_string(s.parameters.size()) + ", "
 	"received " + to_string(nparams));
@@ -919,14 +985,15 @@ pqxx::result pqxx::connection_base::prepared_exec(
       binary[nparams] = 0;
 
       r = result(PQexecPrepared(m_Conn,
-	statement.c_str(),
-	nparams,
-	params,
-	paramlengths,
-	binary.c_ptr(),
-	0),
+		statement.c_str(),
+		nparams,
+		params,
+		paramlengths,
+		binary.c_ptr(),
+		0),
 	protocol_version(),
-	statement);
+	statement,
+	encoding_code());
 
       check_result(r);
       get_notifs();
@@ -940,7 +1007,10 @@ pqxx::result pqxx::connection_base::prepared_exec(
       Q << " (";
       for (int a = 0; a < nparams; ++a)
       {
-	Q << escape_param(params[a],paramlengths[a],s.parameters[a].treatment);
+	Q << escape_param(*this,
+		params[a],
+		paramlengths[a],
+		s.parameters[a].treatment);
 	if (a < nparams-1) Q << ',';
       }
       Q << ')';
@@ -956,10 +1026,12 @@ pqxx::result pqxx::connection_base::prepared_exec(
     for (int n = nparams-1; n >= 0; --n)
     {
       const string key = "$" + to_string(n+1),
-	           val = escape_param(params[n],
+	           val = escape_param(*this,
+				params[n],
 				paramlengths[n],
 				s.parameters[n].treatment);
       const string::size_type keysz = key.size();
+      // TODO: Skip quoted strings!  (And careful with multibyte encodings...)
       for (string::size_type h=S.find(key); h!=string::npos; h=S.find(key))
 	S.replace(h,keysz,val);
     }
@@ -1012,10 +1084,10 @@ void pqxx::connection_base::close() throw ()
       process_notice("Closing connection while " +
 	             m_Trans.get()->description() + " still open");
 
-    if (!m_Triggers.empty())
+    if (!m_listeners.empty())
     {
-      process_notice("Closing connection with outstanding triggers");
-      m_Triggers.clear();
+      process_notice("Closing connection with outstanding listeners");
+      m_listeners.clear();
     }
 
     m_Conn = m_policy.do_disconnect(m_Conn);
@@ -1083,7 +1155,9 @@ void pqxx::connection_base::MakeEmpty(pqxx::result &R)
     throw internal_error("MakeEmpty() on null connection");
 
   R = result(PQmakeEmptyPGresult(m_Conn, PGRES_EMPTY_QUERY),
-             protocol_version());
+		protocol_version(),
+		"[]",
+		encoding_code());
 }
 
 
@@ -1105,17 +1179,17 @@ bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
 
 #ifdef PQXX_HAVE_PQPUTCOPY
   char *Buf = 0;
-  const int proto = protocol_version();
+  const int proto = protocol_version(), encoding=encoding_code();
   const string querydesc = "[END COPY]";
   switch (PQgetCopyData(m_Conn, &Buf, false))
   {
     case -2:
-      throw runtime_error("Reading of table data failed: " + string(ErrMsg()));
+      throw failure("Reading of table data failed: " + string(ErrMsg()));
 
     case -1:
-      for (result R(PQgetResult(m_Conn), proto, querydesc);
+      for (result R(PQgetResult(m_Conn), proto, querydesc, encoding);
            R;
-	   R=result(PQgetResult(m_Conn), proto, querydesc))
+	   R=result(PQgetResult(m_Conn), proto, querydesc, encoding))
 	check_result(R);
       Result = false;
       break;
@@ -1141,7 +1215,7 @@ bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
     {
     case EOF:
       PQendcopy(m_Conn);
-      throw runtime_error("Unexpected EOF from backend");
+      throw failure("Unexpected EOF from backend");
 
     case 0:
       LineComplete = true;
@@ -1151,7 +1225,7 @@ bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
       break;
 
     default:
-      throw runtime_error("Unexpected COPY response from backend");
+      throw failure("Unexpected COPY response from backend");
     }
 
     Line += Buf;
@@ -1167,7 +1241,7 @@ bool pqxx::connection_base::ReadCopyLine(PGSTD::string &Line)
   else
   {
     Line.erase();
-    if (PQendcopy(m_Conn)) throw runtime_error(ErrMsg());
+    if (PQendcopy(m_Conn)) throw failure(ErrMsg());
   }
 #endif
 
@@ -1185,15 +1259,15 @@ void pqxx::connection_base::WriteCopyLine(const PGSTD::string &Line)
   const string::size_type Len = L.size();
 
 #ifdef PQXX_HAVE_PQPUTCOPY
-  if (PQputCopyData(m_Conn, LC, Len) <= 0)
+  if (PQputCopyData(m_Conn, LC, int(Len)) <= 0)
   {
     const string Msg = string("Error writing to table: ") + ErrMsg();
     PQendcopy(m_Conn);
-    throw runtime_error(Msg);
+    throw failure(Msg);
   }
 #else
-  if (PQputnbytes(m_Conn, LC, Len) == EOF)
-    throw runtime_error(string("Error writing to table: ") + ErrMsg());
+  if (PQputnbytes(m_Conn, LC, int(Len)) == EOF)
+    throw failure(string("Error writing to table: ") + ErrMsg());
 #endif
 }
 
@@ -1205,7 +1279,7 @@ void pqxx::connection_base::EndCopyWrite()
   switch (Res)
   {
   case -1:
-    throw runtime_error("Write to table failed: " + string(ErrMsg()));
+    throw failure("Write to table failed: " + string(ErrMsg()));
   case 0:
     throw internal_error("table write is inexplicably asynchronous");
   case 1:
@@ -1217,7 +1291,10 @@ void pqxx::connection_base::EndCopyWrite()
 	"from PQputCopyEnd()");
   }
 
-  const result R(PQgetResult(m_Conn), protocol_version(), "[END COPY]");
+  const result R(PQgetResult(m_Conn),
+		protocol_version(),
+		"[END COPY]",
+		encoding_code());
   check_result(R);
 
 #else
@@ -1226,7 +1303,7 @@ void pqxx::connection_base::EndCopyWrite()
   // returning 1 (i.e., failure) but with an empty error message, and without
   // anything seeming wrong.
   if ((PQendcopy(m_Conn) != 0) && ErrMsg() && *ErrMsg())
-    throw runtime_error(ErrMsg());
+    throw failure(ErrMsg());
 #endif
 }
 
@@ -1234,7 +1311,7 @@ void pqxx::connection_base::EndCopyWrite()
 void pqxx::connection_base::start_exec(const PGSTD::string &Q)
 {
   activate();
-  if (!PQsendQuery(m_Conn, Q.c_str())) throw runtime_error(ErrMsg());
+  if (!PQsendQuery(m_Conn, Q.c_str())) throw failure(ErrMsg());
 }
 
 
@@ -1247,18 +1324,19 @@ pqxx::internal::pq::PGresult *pqxx::connection_base::get_result()
 
 string pqxx::connection_base::esc(const char str[], size_t maxlen)
 {
-#ifdef PQXX_HAVE_PQESCAPESTRINGCONN
+  string escaped;
+
+#if defined(PQXX_HAVE_PQESCAPESTRINGCONN)
   // We need a connection object...  This is the one reason why this function is
   // not const!
   if (!m_Conn) activate();
 
   char *const buf = new char[2*maxlen+1];
-  string escaped;
   try
   {
     int err = 0;
     PQescapeStringConn(m_Conn, buf, str, maxlen, &err);
-    if (err) throw invalid_argument(ErrMsg());
+    if (err) throw argument_error(ErrMsg());
     escaped = string(buf);
   }
   catch (const exception &)
@@ -1268,15 +1346,56 @@ string pqxx::connection_base::esc(const char str[], size_t maxlen)
   }
   delete [] buf;
 
-  return escaped;
+#elif defined(PQXX_HAVE_PQESCAPESTRING)
+  scoped_array<char> buf(new char[2*maxlen+1]);
+  const size_t bytes = PQescapeString(buf.c_ptr(), str, maxlen);
+  escaped.assign(buf.c_ptr(), bytes);
+
 #else
-  return internal::escape_string(str, maxlen);
+  // Last-ditch workaround.  This has serious problems with multibyte encodings.
+  for (size_t i=0; str[i] && (i < maxlen); ++i)
+  {
+    // Ensure we don't pass negative integers to isprint()/isspace(), which
+    // Visual C++ chokes on.
+    const unsigned char c(str[i]);
+    if (c & 0x80)
+    {
+      throw failure("non-ASCII text passed to sqlesc(); "
+	  "the libpq version that libpqxx was built with does not support this "
+	  "yet (minimum is postgres 7.2)");
+    }
+    else if (isprint(c) || isspace(c))
+    {
+      if (c=='\\' || c=='\'') escaped += c;
+      escaped += c;
+    }
+    else
+    {
+        char s[8];
+	// TODO: Number may be formatted according to locale!  :-(
+        sprintf(s, "\\%03o", static_cast<unsigned int>(c));
+        escaped.append(s, 4);
+    }
+  }
 #endif
+
+  return escaped;
 }
 
 
-PGSTD::string pqxx::connection_base::esc_raw(const unsigned char str[],
-	size_t len)
+string pqxx::connection_base::esc(const char str[])
+{
+  return this->esc(str, strlen(str));
+}
+
+
+string pqxx::connection_base::esc(const PGSTD::string &str)
+{
+  return this->esc(str.c_str(), str.size());
+}
+
+
+string pqxx::connection_base::esc_raw(const unsigned char str[], size_t len)
 {
   size_t bytes = 0;
 #ifdef PQXX_HAVE_PQESCAPEBYTEACONN
@@ -1288,7 +1407,7 @@ PGSTD::string pqxx::connection_base::esc_raw(const unsigned char str[],
   if (!buf.c_ptr())
   {
     // TODO: Distinguish different errors in exception type
-    throw runtime_error(ErrMsg());
+    throw failure(ErrMsg());
   }
 #else
   unsigned char *const s = const_cast<unsigned char *>(str);
@@ -1306,8 +1425,8 @@ void wait_fd(int fd, bool forwrite=false, timeval *tv=0)
   if (fd < 0) throw pqxx::broken_connection();
 
 #ifdef PQXX_HAVE_POLL
-  pollfd pfd = { fd, POLLERR|POLLHUP|POLLNVAL | (forwrite?POLLOUT:POLLIN) , 0 };
-  poll(&pfd, 1, (tv ? (tv->tv_sec*1000 + tv->tv_usec/1000) : -1));
+  pollfd pfd = { fd, short(POLLERR|POLLHUP|POLLNVAL | (forwrite?POLLOUT:POLLIN)) , 0 };
+  poll(&pfd, 1, (tv ? int(tv->tv_sec*1000 + tv->tv_usec/1000) : -1));
 #else
   fd_set s;
   clear_fdmask(&s);
@@ -1392,7 +1511,11 @@ void pqxx::connection_base::read_capabilities() throw ()
   {
     // Estimate server version by querying 'version()'.  This may not be exact!
     const string VQ = "SELECT version()";
-    const result r(PQexec(m_Conn, VQ.c_str()), protocol_version(), VQ);
+    const result r(PQexec(m_Conn, VQ.c_str()),
+		protocol_version(),
+		VQ,
+		encoding_code());
+
     int x=0, y=0, z=0;
     if ((sscanf(r[0][0].c_str(), "PostgreSQL %d.%d.%d", &x, &y, &z) == 3) &&
         (x >= 0) && (x < 100) &&
@@ -1432,3 +1555,14 @@ string pqxx::connection_base::adorn_name(const PGSTD::string &n)
   return n.empty() ? ("x"+id) : (n+"_"+id);
 }
 
+
+int pqxx::connection_base::encoding_code() throw ()
+{
+#ifdef PQXX_HAVE_PQENCODING
+  activate();
+  return PQclientEncoding(m_Conn);
+#else
+  return 0;
+#endif
+}
+
