@@ -7,7 +7,7 @@
  *      Implementation of the Large Objects interface
  *   Allows access to large objects directly, or though I/O streams
  *
- * Copyright (c) 2003-2009, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2003-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -30,8 +30,6 @@
 
 #include "pqxx/largeobject"
 
-#include "pqxx/internal/gates/connection-largeobject.hxx"
-
 using namespace PGSTD;
 using namespace pqxx::internal;
 
@@ -137,13 +135,6 @@ void pqxx::largeobject::remove(dbtransaction &T) const
 }
 
 
-pqxx::internal::pq::PGconn *pqxx::largeobject::RawConnection(
-	const dbtransaction &T)
-{
-  return gate::connection_largeobject(T.conn()).RawConnection();
-}
-
-
 string pqxx::largeobject::Reason(int err) const
 {
   if (err == ENOMEM) return "Out of memory";
