@@ -7,7 +7,7 @@
  *      implementation of the pqxx::pipeline class
  *   Throughput-optimized query manager
  *
- * Copyright (c) 2003-2009, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2003-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -20,10 +20,6 @@
 #include "pqxx/dbtransaction"
 #include "pqxx/pipeline"
 
-#include "pqxx/internal/gates/connection-pipeline.hxx"
-#include "pqxx/internal/gates/result-creation.hxx"
-
-
 using namespace PGSTD;
 using namespace pqxx;
 using namespace pqxx::internal;
@@ -141,7 +137,7 @@ void pqxx::pipeline::cancel()
 {
   while (have_pending())
   {
-    gate::connection_pipeline(m_Trans.conn()).cancel_query();
+    m_Trans.conn().cancel_query();
     QueryMap::iterator canceled_query = m_issuedrange.first;
     ++m_issuedrange.first;
     m_queries.erase(canceled_query);
@@ -228,7 +224,7 @@ void pqxx::pipeline::issue()
   const bool prepend_dummy = (num_issued > 1);
   if (prepend_dummy) cum = theDummyQuery + cum;
 
-  gate::connection_pipeline(m_Trans.conn()).start_exec(cum);
+  m_Trans.start_exec(cum);
 
   // Since we managed to send out these queries, update state to reflect this
   m_dummy_pending = prepend_dummy;
@@ -251,8 +247,7 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
   pqxxassert(!m_dummy_pending);
   pqxxassert(!m_queries.empty());
 
-  gate::connection_pipeline gate(m_Trans.conn());
-  internal::pq::PGresult *r = gate.get_result();
+  internal::pq::PGresult *r = m_Trans.get_result();
   if (!r)
   {
     if (have_pending() && !expect_none)
@@ -264,11 +259,10 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
   }
 
   pqxxassert(r);
-  const result res = gate::result_creation::create(
-	r,
-	0,
-	m_queries.begin()->second.get_query(),
-	gate::connection_pipeline(m_Trans.conn()).encoding_code());
+  const result res(r,
+		0,
+		m_queries.begin()->second.get_query(),
+		m_Trans.conn().encoding_code());
 
   if (!have_pending())
   {
@@ -290,23 +284,17 @@ bool pqxx::pipeline::obtain_result(bool expect_none)
 void pqxx::pipeline::obtain_dummy()
 {
   pqxxassert(m_dummy_pending);
-  gate::connection_pipeline gate(m_Trans.conn());
-  internal::pq::PGresult *const r = gate.get_result();
+  internal::pq::PGresult *const r = m_Trans.get_result();
   m_dummy_pending = false;
 
   if (!r)
     internal_error("pipeline got no result from backend when it expected one");
 
-  result R = gate::result_creation::create(
-	r,
-	0,
-	"[DUMMY PIPELINE QUERY]",
-	gate::connection_pipeline(m_Trans.conn()).encoding_code());
-
+  result R(r, 0, "[DUMMY PIPELINE QUERY]", m_Trans.conn().encoding_code());
   bool OK = false;
   try
   {
-    gate::result_creation(R).CheckStatus();
+    R.CheckStatus();
     OK = true;
   }
   catch (const sql_error &)
@@ -358,7 +346,7 @@ void pqxx::pipeline::obtain_dummy()
       const string &query = m_issuedrange.first->second.get_query();
       const result res(m_Trans.exec(query));
       m_issuedrange.first->second.set_result(res);
-      gate::result_creation(res).CheckStatus();
+      res.CheckStatus();
       ++m_issuedrange.first;
     }
     while (m_issuedrange.first != stop);
@@ -431,7 +419,7 @@ pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
 
   m_queries.erase(q);
 
-  gate::result_creation(R).CheckStatus();
+  R.CheckStatus();
   return P;
 }
 
@@ -439,17 +427,15 @@ pqxx::pipeline::retrieve(pipeline::QueryMap::iterator q)
 void pqxx::pipeline::get_further_available_results()
 {
   pqxxassert(!m_dummy_pending);
-  gate::connection_pipeline gate(m_Trans.conn());
-  while (!gate.is_busy() && obtain_result())
-    if (!gate.consume_input()) throw broken_connection();
+  while (!m_Trans.is_busy() && obtain_result())
+    if (!m_Trans.consume_input()) throw broken_connection();
 }
 
 
 void pqxx::pipeline::receive_if_available()
 {
-  gate::connection_pipeline gate(m_Trans.conn());
-  if (!gate.consume_input()) throw broken_connection();
-  if (gate.is_busy()) return;
+  if (!m_Trans.consume_input()) throw broken_connection();
+  if (m_Trans.is_busy()) return;
 
   if (m_dummy_pending) obtain_dummy();
   if (have_pending()) get_further_available_results();
