@@ -7,7 +7,7 @@
  *      Helper classes for defining and executing prepared statements
  *   See the connection_base hierarchy for more about prepared statements
  *
- * Copyright (c) 2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2006-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -64,7 +64,7 @@ pqxx::result pqxx::prepare::invocation::exec() const
     if (m_nonnull[i])
     {
       ptrs[i] = m_values[v].c_str();
-      lens[i] = m_values[v].size();
+      lens[i] = int(m_values[v].size());
       ++v;
     }
     else
@@ -76,7 +76,11 @@ pqxx::result pqxx::prepare::invocation::exec() const
 
   ptrs[elts] = 0;
   lens[elts] = 0;
-  return m_home.prepared_exec(m_statement, ptrs.c_ptr(), lens.c_ptr(), elts);
+  return m_home.prepared_exec(
+	m_statement,
+	 ptrs.c_ptr(),
+	 lens.c_ptr(),
+	 int(elts));
 }
 
 
