@@ -7,7 +7,7 @@
  *      implementation of the pqxx::result class and support classes.
  *   pqxx::result represents the set of result tuples from a database query
  *
- * Copyright (c) 2001-2007, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -17,6 +17,8 @@
  */
 #include "pqxx/compiler-internal.hxx"
 
+#include <cstdlib>
+#include <cstring>
 #include <stdexcept>
 
 #include "libpq-fe.h"
@@ -30,15 +32,22 @@ using namespace PGSTD;
 const string pqxx::result::s_empty_string;
 
 
-pqxx::internal::result_data::result_data() : data(0), protocol(0), query() {}
+pqxx::internal::result_data::result_data() :
+  data(0),
+  protocol(0),
+  query(),
+  encoding_code(0)
+{}
 
 pqxx::internal::result_data::result_data(pqxx::internal::pq::PGresult *d,
 	int p,
-	const PGSTD::string &q) :
+	const PGSTD::string &q,
+	int e) :
   data(d),
   protocol(p),
-  query(q)
-	{}
+  query(q),
+  encoding_code(e)
+{}
 
 
 pqxx::internal::result_data::~result_data() { PQclear(data); }
@@ -49,10 +58,11 @@ void pqxx::internal::freemem_result_data(result_data *d) throw () { delete d; }
 
 pqxx::result::result(pqxx::internal::pq::PGresult *rhs,
 	int protocol,
-	const PGSTD::string &Query) :
-  super(new internal::result_data(rhs, protocol, Query)),
+	const PGSTD::string &Query,
+	int encoding_code) :
+  super(new internal::result_data(rhs, protocol, Query, encoding_code)),
   m_data(rhs)
-	{}
+{}
 
 bool pqxx::result::operator==(const result &rhs) const throw ()
 {
@@ -120,11 +130,9 @@ void pqxx::result::swap(result &rhs) throw ()
 
 
 const pqxx::result::tuple pqxx::result::at(pqxx::result::size_type i) const
-  throw (out_of_range)
+  throw (range_error)
 {
-  if (i >= size())
-    throw out_of_range("Tuple number out of range");
-
+  if (i >= size()) throw range_error("Tuple number out of range");
   return operator[](i);
 }
 
@@ -152,6 +160,11 @@ void pqxx::result::ThrowSQLError(const PGSTD::string &Err,
     case '2':
       throw data_exception(Err, Query);
     case '3':
+      if (strcmp(code,"23001")==0) throw restrict_violation(Err, Query);
+      if (strcmp(code,"23502")==0) throw not_null_violation(Err, Query);
+      if (strcmp(code,"23503")==0) throw foreign_key_violation(Err, Query);
+      if (strcmp(code,"23505")==0) throw unique_violation(Err, Query);
+      if (strcmp(code,"23514")==0) throw check_violation(Err, Query);
       throw integrity_constraint_violation(Err, Query);
     case '4':
       throw invalid_cursor_state(Err, Query);
@@ -187,6 +200,12 @@ void pqxx::result::ThrowSQLError(const PGSTD::string &Err,
       throw insufficient_resources(Err, Query);
     }
     break;
+
+  case 'P':
+    if (strcmp(code, "P0001")==0) throw plpgsql_raise(Err, Query);
+    if (strcmp(code, "P0002")==0) throw plpgsql_no_data_found(Err, Query);
+    if (strcmp(code, "P0003")==0) throw plpgsql_too_many_rows(Err, Query);
+    throw plpgsql_error(Err, Query);
   }
 #endif
   throw sql_error(Err, Query);
@@ -201,8 +220,7 @@ void pqxx::result::CheckStatus() const
 
 string pqxx::result::StatusError() const
 {
-  if (!m_data)
-    throw runtime_error("No result set given");
+  if (!m_data) throw failure("No result set given");
 
   string Err;
 
@@ -246,7 +264,7 @@ const string &pqxx::result::query() const throw ()
 pqxx::oid pqxx::result::inserted_oid() const
 {
   if (!m_data)
-    throw logic_error("Attempt to read oid of inserted row without an INSERT "
+    throw usage_error("Attempt to read oid of inserted row without an INSERT "
 	"result");
   return PQoidValue(m_data);
 }
@@ -262,21 +280,21 @@ pqxx::result::size_type pqxx::result::affected_rows() const
 const char *pqxx::result::GetValue(pqxx::result::size_type Row,
 		                 pqxx::result::tuple::size_type Col) const
 {
-  return PQgetvalue(m_data, Row, Col);
+  return PQgetvalue(m_data, int(Row), int(Col));
 }
 
 
 bool pqxx::result::GetIsNull(pqxx::result::size_type Row,
 		           pqxx::result::tuple::size_type Col) const
 {
-  return PQgetisnull(m_data, Row, Col) != 0;
+  return PQgetisnull(m_data, int(Row), int(Col)) != 0;
 }
 
 pqxx::result::field::size_type
 pqxx::result::GetLength(pqxx::result::size_type Row,
                         pqxx::result::tuple::size_type Col) const
 {
-  return PQgetlength(m_data, Row, Col);
+  return PQgetlength(m_data, int(Row), int(Col));
 }
 
 
@@ -284,7 +302,7 @@ pqxx::oid pqxx::result::column_type(tuple::size_type ColNum) const
 {
   const oid T = PQftype(m_data, ColNum);
   if (T == oid_none)
-    throw PGSTD::invalid_argument(
+    throw argument_error(
 	"Attempt to retrieve type of nonexistant column " +
 	to_string(ColNum) + " of query result");
   return T;
@@ -300,7 +318,7 @@ pqxx::oid pqxx::result::column_table(tuple::size_type ColNum) const
    * got an invalid row number.
    */
   if (T == oid_none && ColNum >= columns())
-    throw PGSTD::invalid_argument("Attempt to retrieve table ID for column " +
+    throw argument_error("Attempt to retrieve table ID for column " +
 	to_string(ColNum) + " out of " + to_string(columns()));
 
   return T;
@@ -321,7 +339,7 @@ pqxx::result::table_column(tuple::size_type ColNum) const
   // 2. Not using protocol 3.0 or better
   // 3. Column not taken directly from a table
   if (ColNum > columns())
-    throw out_of_range("Invalid column index in table_column(): " +
+    throw range_error("Invalid column index in table_column(): " +
       to_string(ColNum));
 
   if (!c_ptr() || c_ptr()->protocol < 3)
@@ -329,7 +347,7 @@ pqxx::result::table_column(tuple::size_type ColNum) const
       "column's original number",
       "[TABLE_COLUMN]");
 
-  throw logic_error("Can't query origin of column " + to_string(ColNum) + ": "
+  throw usage_error("Can't query origin of column " + to_string(ColNum) + ": "
 	"not derived from table column");
 }
 #endif
@@ -360,18 +378,18 @@ pqxx::result::field pqxx::result::tuple::at(const char f[]) const
 {
   const int fnum = m_Home->column_number(f);
   // TODO: Should this be an out_of_range?
-  if (fnum == -1)
-    throw invalid_argument(string("Unknown field '") + f + "'");
+  if (fnum == -1) throw argument_error(string("Unknown field '") + f + "'");
 
   return field(*this, fnum);
 }
 
 
 pqxx::result::field
-pqxx::result::tuple::at(pqxx::result::tuple::size_type i) const throw (out_of_range)
+pqxx::result::tuple::at(pqxx::result::tuple::size_type i) const
+  throw (range_error)
 {
   if (i >= size())
-    throw out_of_range("Invalid field number");
+    throw range_error("Invalid field number");
 
   return operator[](i);
 }
@@ -382,7 +400,7 @@ pqxx::result::column_name(pqxx::result::tuple::size_type Number) const
 {
   const char *const N = PQfname(m_data, Number);
   if (!N)
-    throw out_of_range("Invalid column number: " + to_string(Number));
+    throw range_error("Invalid column number: " + to_string(Number));
 
   return N;
 }
@@ -400,7 +418,7 @@ pqxx::result::column_number(const char ColName[]) const
   const int N = PQfnumber(m_data, ColName);
   // TODO: Should this be an out_of_range?
   if (N == -1)
-    throw invalid_argument("Unknown column name: '" + string(ColName) + "'");
+    throw argument_error("Unknown column name: '" + string(ColName) + "'");
 
   return tuple::size_type(N);
 }
