@@ -78,6 +78,7 @@ void pqxx::subtransaction::do_abort()
 void pqxx::subtransaction::check_backendsupport() const
 {
   if (!m_parent.conn().supports(connection_base::cap_nested_transactions))
-    throw runtime_error("Backend version does not support nested transactions");
+    throw feature_not_supported(
+	"Backend version does not support nested transactions");
 }
 
