@@ -7,7 +7,7 @@
  *      implementation of the pqxx::tablereader class.
  *   pqxx::tablereader enables optimized batch reads from a database table
  *
- * Copyright (c) 2001-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -46,7 +46,7 @@ void pqxx::tablereader::setup(transaction_base &T,
 pqxx::tablereader::~tablereader() throw ()
 {
 #ifdef PQXX_QUIET_DESTRUCTORS
-  internal::disable_noticer Quiet(m_Trans.conn());
+  disable_noticer Quiet(m_Trans.conn());
 #endif
   try
   {
@@ -92,7 +92,7 @@ void pqxx::tablereader::reader_close()
       try
       {
         string Dummy;
-        while (get_raw_line(Dummy));
+        while (get_raw_line(Dummy)) ;
       }
       catch (const broken_connection &)
       {
@@ -121,6 +121,7 @@ inline bool is_octalchar(char o) throw ()
 string::size_type findtab(const PGSTD::string &Line,
 	PGSTD::string::size_type start)
 {
+  // TODO: Fix for multibyte encodings?
   const string::size_type here = Line.find('\t', start);
   return (here == string::npos) ? Line.size() : here;
 }
@@ -148,13 +149,13 @@ string pqxx::tablereader::extract_field(const PGSTD::string &Line,
       {
         const char n = Line[++i];
         if (i >= Line.size())
-          throw runtime_error("Row ends in backslash");
+          throw failure("Row ends in backslash");
 
 	switch (n)
 	{
 	case 'N':	// Null value
 	  if (!R.empty())
-	    throw runtime_error("Null sequence found in nonempty field");
+	    throw failure("Null sequence found in nonempty field");
 	  R = NullStr();
 	  isnull = true;
 	  break;
@@ -169,11 +170,11 @@ string pqxx::tablereader::extract_field(const PGSTD::string &Line,
 	case '7':
           {
 	    if ((i+2) >= Line.size())
-	      throw runtime_error("Row ends in middle of octal value");
+	      throw failure("Row ends in middle of octal value");
 	    const char n1 = Line[++i];
 	    const char n2 = Line[++i];
 	    if (!is_octalchar(n1) || !is_octalchar(n2))
-	      throw runtime_error("Invalid octal in encoded table stream");
+	      throw failure("Invalid octal in encoded table stream");
 	    R += char((digit_to_number(n)<<6) |
 		(digit_to_number(n1)<<3) |
 		digit_to_number(n2));
@@ -224,7 +225,7 @@ string pqxx::tablereader::extract_field(const PGSTD::string &Line,
   ++i;
 
   if (isnull && (R.size() != NullStr().size()))
-    throw runtime_error("Field contains data behind null sequence");
+    throw failure("Field contains data behind null sequence");
 
   return R;
 }
