@@ -7,7 +7,7 @@
  *      implementation of the pqxx::transaction class.
  *   pqxx::transaction represents a regular database transaction
  *
- * Copyright (c) 2001-2005, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -28,7 +28,7 @@ using namespace PGSTD;
 
 
 pqxx::basic_transaction::basic_transaction(connection_base &C,
-    const string &IsolationLevel) :
+    const PGSTD::string &IsolationLevel) :
   namedclass("transaction"),
   dbtransaction(C, IsolationLevel)
 {
