@@ -8,7 +8,7 @@
  *   pqxx::transaction_base defines the interface for any abstract class that
  *   represents a database transaction
  *
- * Copyright (c) 2001-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2001-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -18,6 +18,7 @@
  */
 #include "pqxx/compiler-internal.hxx"
 
+#include <cstring>
 #include <stdexcept>
 
 #include "pqxx/connection_base"
@@ -93,7 +94,7 @@ void pqxx::transaction_base::commit()
     break;
 
   case st_aborted:
-    throw logic_error("Attempt to commit previously aborted " + description());
+    throw usage_error("Attempt to commit previously aborted " + description());
 
   case st_committed:
     // Transaction has been committed already.  This is not exactly proper
@@ -107,7 +108,7 @@ void pqxx::transaction_base::commit()
   case st_in_doubt:
     // Transaction may or may not have been committed.  Report the problem but
     // don't compound our troubles by throwing.
-    throw logic_error(description() +
+    throw usage_error(description() +
 		      "committed again while in an indeterminate state");
 
   default:
@@ -119,9 +120,9 @@ void pqxx::transaction_base::commit()
   // commit is premature.  Punish this swiftly and without fail to discourage
   // the habit from forming.
   if (m_Focus.get())
-    throw runtime_error("Attempt to commit " + description() + " "
-			"with " + m_Focus.get()->description() + " "
-			"still open");
+    throw failure("Attempt to commit " + description() + " "
+		"with " + m_Focus.get()->description() + " "
+		"still open");
 
   // Check that we're still connected (as far as we know--this is not an
   // absolute thing!) before trying to commit.  If the connection was broken
@@ -170,7 +171,7 @@ void pqxx::transaction_base::abort()
     return;
 
   case st_committed:
-    throw logic_error("Attempt to abort previously committed " + description());
+    throw usage_error("Attempt to abort previously committed " + description());
 
   case st_in_doubt:
     // Aborting an in-doubt transaction is probably a reasonably sane response
@@ -189,18 +190,6 @@ void pqxx::transaction_base::abort()
 }
 
 
-string pqxx::transaction_base::esc(const char str[]) const
-{
-  return m_Conn.esc(str, strlen(str));
-}
-
-
-string pqxx::transaction_base::esc(const PGSTD::string &str) const
-{
-  return m_Conn.esc(str.c_str(), str.size());
-}
-
-
 string pqxx::transaction_base::esc_raw(const PGSTD::string &str) const
 {
   const unsigned char *p = reinterpret_cast<const unsigned char *>(str.c_str());
@@ -208,19 +197,8 @@ string pqxx::transaction_base::esc_raw(const PGSTD::string &str) const
 }
 
 
-pqxx::result pqxx::transaction_base::exec(const PGSTD::string &Query,
-					  const PGSTD::string &Desc)
+void pqxx::transaction_base::activate()
 {
-  CheckPendingError();
-
-  const string N = (Desc.empty() ? "" : "'" + Desc + "' ");
-
-  if (m_Focus.get())
-    throw logic_error("Attempt to execute query " + N +
-		      "on " + description() + " "
-		      "with " + m_Focus.get()->description() + " "
-		      "still open");
-
   switch (m_Status)
   {
   case st_nascent:
@@ -234,12 +212,37 @@ pqxx::result pqxx::transaction_base::exec(const PGSTD::string &Query,
   case st_committed:
   case st_aborted:
   case st_in_doubt:
-    throw logic_error("Attempt to execute query " + N + " "
-	"in " + description() + ", which is already closed");
+    throw usage_error(
+	"Attempt to activate " + description() + " "
+	"which is already closed");
 
   default:
     throw internal_error("pqxx::transaction: invalid status code");
   }
+}
+
+
+pqxx::result pqxx::transaction_base::exec(const PGSTD::string &Query,
+					  const PGSTD::string &Desc)
+{
+  CheckPendingError();
+
+  const string N = (Desc.empty() ? "" : "'" + Desc + "' ");
+
+  if (m_Focus.get())
+    throw usage_error("Attempt to execute query " + N +
+		      "on " + description() + " "
+		      "with " + m_Focus.get()->description() + " "
+		      "still open");
+
+  try
+  {
+    activate();
+  }
+  catch (const usage_error &e)
+  {
+    throw usage_error("Error executing query " + N + ".  " + e.what());
+  }
 
   // TODO: Pass Desc to do_exec(), and from there on down
   return do_exec(Query.c_str());
@@ -249,6 +252,15 @@ pqxx::result pqxx::transaction_base::exec(const PGSTD::string &Query,
 pqxx::prepare::invocation
 pqxx::transaction_base::prepared(const PGSTD::string &statement)
 {
+  try
+  {
+    activate();
+  }
+  catch (const usage_error &e)
+  {
+    throw usage_error(
+	"Error executing prepared statement " + statement + ".  " + e.what());
+  }
   return prepare::invocation(*this, statement);
 }
 
@@ -326,12 +338,7 @@ void pqxx::transaction_base::End() throw ()
     try { abort(); }
     catch (const exception &e) { m_Conn.process_notice(e.what()); }
 
-    const int ra = m_reactivation_avoidance.get();
-    if (ra)
-    {
-      reactivation_avoidance_clear();
-      conn().m_reactivation_avoidance.add(ra);
-    }
+    m_reactivation_avoidance.give_to(conn().m_reactivation_avoidance);
   }
   catch (const exception &e)
   {
@@ -404,7 +411,7 @@ void pqxx::transaction_base::CheckPendingError()
 #else
     m_PendingError.resize(0);
 #endif
-    throw runtime_error(m_PendingError);
+    throw failure(m_PendingError);
   }
 }
 
