@@ -6,7 +6,7 @@
  *   DESCRIPTION
  *      Various utility functions for libpqxx
  *
- * Copyright (c) 2003-2006, Jeroen T. Vermeulen <jtv@xs4all.nl>
+ * Copyright (c) 2003-2008, Jeroen T. Vermeulen <jtv@xs4all.nl>
  *
  * See COPYING for copyright license.  If you did not receive a file called
  * COPYING with this source code, please notify the distributor of this mistake,
@@ -19,6 +19,10 @@
 #include <cmath>
 #include <cstring>
 
+#ifdef PQXX_HAVE_LIMITS
+#include <limits>
+#endif
+
 #ifdef PQXX_HAVE_LOCALE
 #include <locale>
 #endif
@@ -79,14 +83,26 @@ template<> inline void set_to_NaN(double &t) { t = nan_d; }
 const long double nan_ld(0.0/0.0);
 template<> inline void set_to_NaN(long double &t) { t = nan_ld; }
 #endif
-
 #endif
 
+// TODO: This may need tweaking for various compilers.
+template<typename T> inline void set_to_Inf(T &t, int sign=1)
+{
+#ifdef PQXX_HAVE_LIMITS
+  T value = numeric_limits<T>::infinity();
+#else
+  T value = INFINITY;
+#endif
+  if (sign < 0) value = -value;
+  t = value;
+}
+
+
 /// For use in string parsing: add new numeric digit to intermediate value
 template<typename L, typename R>
   inline L absorb_digit(L value, R digit) throw ()
 {
-  return 10*value + digit;
+  return L(L(10)*value + L(digit));
 }
 
 
@@ -98,14 +114,14 @@ template<typename T> void from_string_signed(const char Str[], T &Obj)
   if (!isdigit(Str[i]))
   {
     if (Str[i] != '-')
-      throw runtime_error("Could not convert string to integer: '" +
+      throw pqxx::failure("Could not convert string to integer: '" +
 	string(Str) + "'");
 
     for (++i; isdigit(Str[i]); ++i)
     {
       const T newresult = absorb_digit(result, -digit_to_number(Str[i]));
       if (newresult > result)
-        throw runtime_error("Integer too small to read: " + string(Str));
+        throw pqxx::failure("Integer too small to read: " + string(Str));
       result = newresult;
     }
   }
@@ -113,12 +129,12 @@ template<typename T> void from_string_signed(const char Str[], T &Obj)
   {
     const T newresult = absorb_digit(result, digit_to_number(Str[i]));
     if (newresult < result)
-      throw runtime_error("Integer too large to read: " + string(Str));
+      throw pqxx::failure("Integer too large to read: " + string(Str));
     result = newresult;
   }
 
   if (Str[i])
-    throw runtime_error("Unexpected text after integer: '" + string(Str) + "'");
+    throw pqxx::failure("Unexpected text after integer: '" + string(Str) + "'");
 
   Obj = result;
 }
@@ -126,30 +142,38 @@ template<typename T> void from_string_signed(const char Str[], T &Obj)
 template<typename T> void from_string_unsigned(const char Str[], T &Obj)
 {
   int i = 0;
-  T result;
-
-  if (!Str) throw runtime_error("Attempt to convert NULL string to integer");
+  T result = 0;
 
   if (!isdigit(Str[i]))
-    throw runtime_error("Could not convert string to unsigned integer: '" +
+    throw pqxx::failure("Could not convert string to unsigned integer: '" +
 	string(Str) + "'");
 
-  for (result=0; isdigit(Str[i]); ++i)
+  for (; isdigit(Str[i]); ++i)
   {
     const T newres = absorb_digit(result, digit_to_number(Str[i]));
     if (newres < result)
-      throw runtime_error("Unsigned integer too large to read: " + string(Str));
+      throw pqxx::failure("Unsigned integer too large to read: " + string(Str));
 
     result = newres;
   }
 
   if (Str[i])
-    throw runtime_error("Unexpected text after integer: '" + string(Str) + "'");
+    throw pqxx::failure("Unexpected text after integer: '" + string(Str) + "'");
 
   Obj = result;
 }
 
 
+bool valid_infinity_string(const char str[])
+{
+  // TODO: Also accept less sensible case variations.
+  return
+	strcmp("infinity", str) == 0 ||
+	strcmp("Infinity", str) == 0 ||
+	strcmp("INFINITY", str) == 0;
+}
+
+
 /* These are hard.  Sacrifice performance of specialized, nonflexible,
  * non-localized code and lean on standard library.  Some special-case code
  * handles NaNs.
@@ -159,159 +183,45 @@ template<typename T> inline void from_string_float(const char Str[], T &Obj)
   bool ok = false;
   T result;
 
-  if (Str[0]=='N' || Str[0]=='n')
+  switch (Str[0])
   {
+  case 'N':
+  case 'n':
     // Accept "NaN," "nan," etc.
     ok = ((Str[1]=='A'||Str[1]=='a') && (Str[2]=='N'||Str[2]=='n') && !Str[3]);
     set_to_NaN(result);
-  }
-  else
-  {
-    stringstream S(Str);
+    break;
+
+  case 'I':
+  case 'i':
+    ok = valid_infinity_string(Str);
+    set_to_Inf(result);
+    break;
+
+  default:
+    if (Str[0] == '-' && valid_infinity_string(&Str[1]))
+    {
+      ok = true;
+      set_to_Inf(result, -1);
+    }
+    else
+    {
+      stringstream S(Str);
 #if defined(PQXX_HAVE_IMBUE)
-    S.imbue(locale("C"));
+      S.imbue(locale("C"));
 #endif
-    ok = (S >> result);
+      ok = (S >> result);
+    }
+    break;
   }
 
   if (!ok)
-    throw runtime_error("Could not convert string to numeric value: '" +
+    throw pqxx::failure("Could not convert string to numeric value: '" +
 	string(Str) + "'");
 
   Obj = result;
 }
 
-} // namespace
-
-
-namespace pqxx
-{
-template<> void from_string(const char Str[], PGSTD::string &Obj, size_t len)
-{
-  Obj = string(Str, len);
-}
-
-template<> void from_string(const char Str[], long &Obj)
-{
-  from_string_signed(Str, Obj);
-}
-
-template<> void from_string(const char Str[], unsigned long &Obj)
-{
-  from_string_unsigned(Str, Obj);
-}
-
-template<> void from_string(const char Str[], int &Obj)
-{
-  from_string_signed(Str, Obj);
-}
-
-template<> void from_string(const char Str[], unsigned int &Obj)
-{
-  from_string_unsigned(Str, Obj);
-}
-
-template<> void from_string(const char Str[], short &Obj)
-{
-  from_string_signed(Str, Obj);
-}
-
-template<> void from_string(const char Str[], unsigned short &Obj)
-{
-  from_string_unsigned(Str, Obj);
-}
-
-#ifdef PQXX_HAVE_LONG_LONG
-template<> void from_string(const char Str[], long long &Obj)
-{
-  from_string_signed(Str, Obj);
-}
-
-template<> void from_string(const char Str[], unsigned long long &Obj)
-{
-  from_string_unsigned(Str, Obj);
-}
-#endif
-
-template<> void from_string(const char Str[], float &Obj)
-{
-  float result;
-  from_string_float(Str, result);
-  Obj = result;
-}
-
-template<> void from_string(const char Str[], double &Obj)
-{
-  from_string_float(Str, Obj);
-}
-
-#if defined(PQXX_HAVE_LONG_DOUBLE)
-template<> void from_string(const char Str[], long double &Obj)
-{
-  from_string_float(Str, Obj);
-}
-#endif
-
-
-template<> void from_string(const char Str[], bool &Obj)
-{
-  if (!Str)
-    throw runtime_error("Attempt to read NULL string");
-
-  bool OK, result=false;
-
-  switch (Str[0])
-  {
-  case 0:
-    result = false;
-    OK = true;
-    break;
-
-  case 'f':
-  case 'F':
-    result = false;
-    OK = !(Str[1] &&
-	   (strcmp(Str+1, "alse") != 0) &&
-	   (strcmp(Str+1, "ALSE") != 0));
-    break;
-
-  case '0':
-    {
-      int I;
-      from_string(Str, I);
-      result = (I != 0);
-      OK = ((I == 0) || (I == 1));
-    }
-    break;
-
-  case '1':
-    result = true;
-    OK = !Str[1];
-    break;
-
-  case 't':
-  case 'T':
-    result = true;
-    OK = !(Str[1] &&
-	   (strcmp(Str+1, "rue") != 0) &&
-	   (strcmp(Str+1, "RUE") != 0));
-    break;
-
-  default:
-    OK = false;
-  }
-
-  if (!OK)
-    throw invalid_argument("Failed conversion to bool: '" + string(Str) + "'");
-
-  Obj = result;
-}
-
-} // namespace pqxx
-
-
-namespace
-{
 template<typename T> inline string to_string_unsigned(T Obj)
 {
   if (!Obj) return "0";
@@ -322,10 +232,10 @@ template<typename T> inline string to_string_unsigned(T Obj)
 
   char *p = &buf[sizeof(buf)];
   *--p = '\0';
-  for (T next; Obj > 0; Obj = next)
+  while (Obj > 0)
   {
-    next = Obj / 10;
-    *--p = number_to_digit(Obj-next*10);
+    *--p = number_to_digit(int(Obj%10));
+    Obj /= 10;
   }
   return p;
 }
@@ -336,18 +246,26 @@ template<typename T> inline string to_string_fallback(T Obj)
 #ifdef PQXX_HAVE_IMBUE
   S.imbue(locale("C"));
 #endif
+
+// Provide enough precision.
+#ifdef PQXX_HAVE_LIMITS
+  // Kirit reports getting two more digits of precision than
+  // numeric_limits::digits10 would give him, so we try not to make him lose
+  // those last few bits.
+  S.precision(numeric_limits<T>::digits10 + 2);
+#else
+  // Guess: enough for an IEEE 754 double-precision value.
+  S.precision(16);
+#endif
   S << Obj;
-  string R;
-  S >> R;
-  return R;
+  return S.str();
 }
 
 
 template<typename T> inline bool is_NaN(T Obj)
 {
-    // TODO: Is this workaround for missing isnan() reliable?  Infinities?
   return
-#if defined(PQXX_HAVE_ISNAN)
+#if defined(PQXX_HAVE_C_ISNAN)
     isnan(Obj);
 #elif defined(PQXX_HAVE_LIMITS)
     !(Obj <= Obj+numeric_limits<T>::max());
@@ -357,11 +275,26 @@ template<typename T> inline bool is_NaN(T Obj)
 }
 
 
+template<typename T> inline bool is_Inf(T Obj)
+{
+  return
+#if defined(PQXX_HAVE_C_ISINF)
+    isinf(Obj);
+#else
+    Obj >= Obj+1 && Obj <= 2*Obj && Obj >= 2*Obj;
+#endif
+}
+
+
 template<typename T> inline string to_string_float(T Obj)
 {
   // TODO: Omit this special case if NaN is output as "nan"/"NAN"/"NaN"
 #ifndef PQXX_HAVE_NAN_OUTPUT
   if (is_NaN(Obj)) return "nan";
+#endif
+  // TODO: Omit this special case if infinity is output as "infinity"
+#ifndef PQXX_HAVE_INF_OUTPUT
+  if (is_Inf(Obj)) return Obj > 0 ? "infinity" : "-infinity";
 #endif
   return to_string_fallback(Obj);
 }
@@ -393,124 +326,195 @@ template<typename T> inline string to_string_signed(T Obj)
 
 namespace pqxx
 {
-template<> string to_string(const short &Obj)
+
+namespace internal
 {
-  return to_string_signed(Obj);
+void throw_null_conversion(const PGSTD::string &type)
+{
+  throw conversion_error("Attempt to convert null to " + type);
+}
+} // namespace pqxx::internal
+
+void string_traits<bool>::from_string(const char Str[], bool &Obj)
+{
+  bool OK, result=false;
+
+  switch (Str[0])
+  {
+  case 0:
+    result = false;
+    OK = true;
+    break;
+
+  case 'f':
+  case 'F':
+    result = false;
+    OK = !(Str[1] &&
+	   (strcmp(Str+1, "alse") != 0) &&
+	   (strcmp(Str+1, "ALSE") != 0));
+    break;
+
+  case '0':
+    {
+      int I;
+      string_traits<int>::from_string(Str, I);
+      result = (I != 0);
+      OK = ((I == 0) || (I == 1));
+    }
+    break;
+
+  case '1':
+    result = true;
+    OK = !Str[1];
+    break;
+
+  case 't':
+  case 'T':
+    result = true;
+    OK = !(Str[1] &&
+	   (strcmp(Str+1, "rue") != 0) &&
+	   (strcmp(Str+1, "RUE") != 0));
+    break;
+
+  default:
+    OK = false;
+  }
+
+  if (!OK)
+    throw argument_error("Failed conversion to bool: '" + string(Str) + "'");
+
+  Obj = result;
 }
 
-template<> string to_string(const unsigned short &Obj)
-{
-  return to_string_unsigned(Obj);
-}
-
-template<> string to_string(const int &Obj)
-{
-  return to_string_signed(Obj);
-}
-
-template<> string to_string(const unsigned int &Obj)
-{
-  return to_string_unsigned(Obj);
-}
-
-template<> string to_string(const long &Obj)
-{
-  return to_string_signed(Obj);
-}
-
-template<> string to_string(const unsigned long &Obj)
-{
-  return to_string_unsigned(Obj);
-}
-
-
-#if defined(PQXX_HAVE_LONG_LONG)
-template<> string to_string(const long long &Obj)
-{
-  return to_string_signed(Obj);
-}
-template<> string to_string(const unsigned long long &Obj)
-{
-  return to_string_unsigned(Obj);
-}
-#endif
-
-template<> string to_string(const float &Obj)
-{
-  return to_string_float(Obj);
-}
-
-template<> string to_string(const double &Obj)
-{
-  return to_string_float(Obj);
-}
-
-#if defined(PQXX_HAVE_LONG_DOUBLE)
-template<> string to_string(const long double &Obj)
-{
-  return to_string_float(Obj);
-}
-#endif
-
-template<> string to_string(const bool &Obj)
+string string_traits<bool>::to_string(bool Obj)
 {
   return Obj ? "true" : "false";
 }
 
-template<> string to_string(const char &Obj)
+void string_traits<short>::from_string(const char Str[], short &Obj)
 {
-  string s;
-  s += Obj;
-  return s;
+  from_string_signed(Str, Obj);
 }
 
-} // namespace pqxx
-
-
-string pqxx::internal::escape_string(const char str[], size_t maxlen)
+string string_traits<short>::to_string(short Obj)
 {
-  string result;
+  return to_string_signed(Obj);
+}
 
-#ifdef PQXX_HAVE_PQESCAPESTRING
-  scoped_array<char> buf;
+void string_traits<unsigned short>::from_string(
+	const char Str[],
+	unsigned short &Obj)
+{
+  from_string_unsigned(Str, Obj);
+}
 
-  buf = new char[2*maxlen+1];
+string string_traits<unsigned short>::to_string(unsigned short Obj)
+{
+  return to_string_unsigned(Obj);
+}
 
-  const size_t bytes = PQescapeString(buf.c_ptr(), str, maxlen);
-  result.assign(buf.c_ptr(), bytes);
+void string_traits<int>::from_string(const char Str[], int &Obj)
+{
+  from_string_signed(Str, Obj);
+}
 
-#else
-  // This is problematic.  What do we do for multibyte encoding?  What do we do
-  // for encodings different from our own, where conversion may be needed?
-  for (size_t i=0; str[i] && (i < maxlen); ++i)
-  {
-    // Ensure we don't pass negative integers to isprint()/isspace(), which
-    // Visual C++ chokes on.
-    const unsigned char c(str[i]);
-    if (c & 0x80)
-    {
-      throw runtime_error("non-ASCII text passed to sqlesc(); "
-	  "the libpq version that libpqxx was built with does not support this "
-	  "yet (minimum is postgres 7.2)");
-    }
-    else if (isprint(c))
-    {
-      if (c=='\\' || c=='\'') result += c;
-      result += c;
-    }
-    else
-    {
-        char s[8];
-	// TODO: Number may be formatted according to locale!  :-(
-        sprintf(s, "\\%03o", static_cast<unsigned int>(c));
-        result.append(s, 4);
-    }
-  }
+string string_traits<int>::to_string(int Obj)
+{
+  return to_string_signed(Obj);
+}
+
+void string_traits<unsigned int>::from_string(
+	const char Str[],
+	unsigned int &Obj)
+{
+  from_string_unsigned(Str, Obj);
+}
+
+string string_traits<unsigned int>::to_string(unsigned int Obj)
+{
+  return to_string_unsigned(Obj);
+}
+
+void string_traits<long>::from_string(const char Str[], long &Obj)
+{
+  from_string_signed(Str, Obj);
+}
+
+string string_traits<long>::to_string(long Obj)
+{
+  return to_string_signed(Obj);
+}
+
+void string_traits<unsigned long>::from_string(
+	const char Str[],
+	unsigned long &Obj)
+{
+  from_string_unsigned(Str, Obj);
+}
+
+string string_traits<unsigned long>::to_string(unsigned long Obj)
+{
+  return to_string_unsigned(Obj);
+}
+
+#ifdef PQXX_HAVE_LONG_LONG
+void string_traits<long long>::from_string(const char Str[], long long &Obj)
+{
+  from_string_signed(Str, Obj);
+}
+
+string string_traits<long long>::to_string(long long Obj)
+{
+  return to_string_signed(Obj);
+}
+
+void string_traits<unsigned long long>::from_string(
+	const char Str[],
+	unsigned long long &Obj)
+{
+  from_string_unsigned(Str, Obj);
+}
+
+string string_traits<unsigned long long>::to_string(unsigned long long Obj)
+{
+  return to_string_unsigned(Obj);
+}
 #endif
 
-  return result;
+void string_traits<float>::from_string(const char Str[], float &Obj)
+{
+  from_string_float(Str, Obj);
 }
 
+string string_traits<float>::to_string(float Obj)
+{
+  return to_string_float(Obj);
+}
+
+void string_traits<double>::from_string(const char Str[], double &Obj)
+{
+  from_string_float(Str, Obj);
+}
+
+string string_traits<double>::to_string(double Obj)
+{
+  return to_string_float(Obj);
+}
+
+#ifdef PQXX_HAVE_LONG_DOUBLE
+void string_traits<long double>::from_string(const char Str[], long double &Obj)
+{
+  from_string_float(Str, Obj);
+}
+
+string string_traits<long double>::to_string(long double Obj)
+{
+  return to_string_float(Obj);
+}
+#endif
+
+} // namespace pqxx
+
 
 namespace
 {
@@ -520,28 +524,12 @@ size_t pqxx_strnlen(const char s[], size_t max)
   return strnlen(s,max);
 #else
   size_t len;
-  for (len=0; len<max && s[len]; ++len);
+  for (len=0; len<max && s[len]; ++len) ;
   return len;
 #endif
 }
 } // namespace
 
-string pqxx::sqlesc(const char str[])
-{
-  return escape_string(str, strlen(str));
-}
-
-string pqxx::sqlesc(const char str[], size_t maxlen)
-{
-  return escape_string(str, pqxx_strnlen(str,maxlen));
-}
-
-
-string pqxx::sqlesc(const string &str)
-{
-  return sqlesc(str.c_str(), str.size());
-}
-
 
 void pqxx::internal::freemem_notif(pqxx::internal::pq::PGnotify *p) throw ()
 {
@@ -611,8 +599,8 @@ void pqxx::internal::CheckUniqueRegistration(const namedclass *New,
   if (Old)
   {
     if (Old == New)
-      throw logic_error("Started twice: " + New->description());
-    throw logic_error("Started " + New->description() + " "
+      throw usage_error("Started twice: " + New->description());
+    throw usage_error("Started " + New->description() + " "
 		      "while " + Old->description() + " still active");
   }
 }
@@ -624,11 +612,11 @@ void pqxx::internal::CheckUniqueUnregistration(const namedclass *New,
   if (New != Old)
   {
     if (!New)
-      throw logic_error("Expected to close " + Old->description() + ", "
+      throw usage_error("Expected to close " + Old->description() + ", "
 			"but got NULL pointer instead");
     if (!Old)
-      throw logic_error("Closed while not open: " + New->description());
-    throw logic_error("Closed " + New->description() + "; "
+      throw usage_error("Closed while not open: " + New->description());
+    throw usage_error("Closed " + New->description() + "; "
 		      "expected to close " + Old->description());
   }
 }
@@ -664,7 +652,7 @@ void pqxx::internal::sleep_seconds(int s)
   if (select(0, &F, &F, &F, &timeout) == -1) switch (errno)
   {
   case EINVAL:	// Invalid timeout
-	throw out_of_range("Invalid timeout value: " + to_string(s));
+	throw range_error("Invalid timeout value: " + to_string(s));
   case EINTR:	// Interrupted by signal
 	break;
   case ENOMEM:	// Out of memory
@@ -678,10 +666,15 @@ void pqxx::internal::sleep_seconds(int s)
 
 namespace
 {
-void copymsg(char buf[], const char msg[], size_t buflen) throw ()
+
+void cpymsg(char buf[], const char input[], size_t buflen) throw ()
 {
-  strncpy(buf, msg, buflen);
-  if (strlen(msg) >= buflen) buf[buflen-1] = '\0';
+#if defined(PQXX_HAVE_STRLCPY)
+  strlcpy(buf, input, buflen);
+#else
+  strncpy(buf, input, buflen);
+  if (buflen) buf[buflen-1] = '\0';
+#endif
 }
 
 // Single Unix Specification version of strerror_r returns result code
@@ -690,9 +683,9 @@ const char *strerror_r_result(int sus_return, char buf[], size_t len) throw ()
   switch (sus_return)
   {
   case 0: break;
-  case -1: copymsg(buf, "Unknown error", len); break;
+  case -1: cpymsg(buf, "Unknown error", len); break;
   default:
-    copymsg(buf,
+    cpymsg(buf,
 	  "Unexpected result from strerror_r()!  Is it really SUS-compliant?",
 	  len);
     break;
@@ -717,7 +710,7 @@ const char *pqxx::internal::strerror_wrapper(int err, char buf[], size_t len)
   const char *res = buf;
 
 #if !defined(PQXX_HAVE_STRERROR_R)
-  copymsg(buf, strerror(err), len);
+  cpymsg(buf, strerror(err), len);
 #else
   // This will pick the appropriate strerror_r() subwrapper using overloading
   // (an idea first suggested by Bart Samwel.  Thanks a bundle, Bart!)
